"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = __importDefault(require("../../../logger"));
/**
 * The current version of this sdk use `node-auth0` v3. But `SCIM` features are not natively supported by v3.
 * This is a workaround to make this SDK support SCIM without `node-auth0` upgrade.
 */
class ScimHandler {
    constructor(config, connectionsManager, poolClient) {
        this.scimStrategies = ['samlp', 'oidc', 'okta', 'waad'];
        this.scimScopes = { read: true, create: true, update: true, delete: true };
        this.scopeMethodMap = {
            get: 'read',
            post: 'create',
            patch: 'update',
            delete: 'delete',
        };
        this.config = config;
        this.connectionsManager = connectionsManager;
        this.poolClient = poolClient;
        this.idMap = new Map();
    }
    /**
     * Check if the connection strategy is SCIM supported.
     * Only few of the enterprise connections are SCIM supported.
     */
    isScimStrategy(strategy) {
        return this.scimStrategies.includes(strategy.toLowerCase());
    }
    /**
     * Creates connection_id -> { strategy, scimConfiguration } map.
     * Store only the SCIM ids available on the existing / remote config.
     * Payload received on `create` and `update` methods has the property `strategy` stripped.
     * So, we need this map to perform `create`, `update` or `delete` actions on SCIM.
     * @param connections
     */
    createIdMap(connections) {
        return __awaiter(this, void 0, void 0, function* () {
            this.idMap.clear();
            logger_1.default.info('Reviewing connections for SCIM support. This may take a while...');
            yield this.poolClient
                .addEachTask({
                data: connections || [],
                generator: (connection) => {
                    if (!this.scimScopes.read)
                        return Promise.resolve(null);
                    if (!this.isScimStrategy(connection.strategy))
                        return Promise.resolve(null);
                    this.idMap.set(connection.id, { strategy: connection.strategy });
                    return this.getScimConfiguration({ id: connection.id })
                        .then((response) => {
                        const scimConfiguration = response === null || response === void 0 ? void 0 : response.data;
                        if (scimConfiguration) {
                            const { mapping, user_id_attribute, connection_id } = scimConfiguration;
                            this.idMap.set(connection_id, Object.assign(Object.assign({}, this.idMap.get(connection_id)), { scimConfiguration: { mapping, user_id_attribute } }));
                        }
                    })
                        .catch((error) => {
                        throw new Error(`Problem fetching SCIM configurations while running \"createIdMap\".\n${error}`);
                    });
                },
            })
                .promise();
        });
    }
    /**
     * Iterate through all the connections and add property `scim_configuration` to only `SCIM` connections.
     * The following conditions should be met to have `scim_configuration` set to a `connection`.
     * 1. Connection `strategy` should be one of `scimStrategies`
     * 2. Connection should have `SCIM` enabled.
     *
     * This method mutates the incoming `connections`.
     */
    applyScimConfiguration(connections) {
        return __awaiter(this, void 0, void 0, function* () {
            // If `this.idMap` is empty, it means we haven't created the idMap yet. Create it.
            // If `this.scimScopes.read` is false, it means we don't have `read:scim_config` scope. Return connections as is.
            if (this.idMap.size === 0) {
                if (!this.scimScopes.read)
                    return connections;
                yield this.createIdMap(connections);
            }
            for (const connection of connections) {
                const { scimConfiguration } = this.idMap.get(connection.id) || {};
                if (scimConfiguration) {
                    connection.scim_configuration = scimConfiguration;
                }
            }
        });
    }
    /**
     * Wrapper over scimClient methods.
     */
    /*
    private async useScimClient(
      method: string,
      options: [ScimRequestParams, ...Record<string, any>[]]
    ): Promise<Asset> {
      return await this.withErrorHandling(
        async () => {
          return await this.scimClient[method](...options);
        },
        method,
        options[0].id
      );
    }
    */
    /**
     * Error handler wrapper.
     */
    withErrorHandling(callback, method, connectionId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield callback();
            }
            catch (error) {
                return this.handleExpectedErrors(error, method, connectionId);
            }
        });
    }
    /**
     * Handle expected errors.
     */
    handleExpectedErrors(error, method, connectionId) {
        var _a;
        // Skip the connection if it returns 404. This can happen if `SCIM` is not enabled on a `SCIM` connection.
        if (error && error.statusCode === 404) {
            logger_1.default.debug(`SCIM configuration is not enabled on connection \"${connectionId}\".`);
            return null;
        }
        // Skip the connection if it returns 403. Looks like "scim_config" permissions are not enabled on Management API.
        if (error && error.statusCode === 403) {
            const scope = this.scopeMethodMap[method];
            this.scimScopes[scope] = false;
            const warningMessage = `Insufficient scope, \"${scope}:scim_config\". Required \"read:scim_config\", \"create:scim_config\", \"update:scim_config\" and \"delete:scim_config\".`;
            const suggestionText = 'If you are not using SCIM, you can keep these permissions disabled and ignore this warning.';
            logger_1.default.warn(`${warningMessage}\n${suggestionText}\n`);
            return null;
        }
        // Skip the connection if it returns 400. This can happen if `SCIM` configuration already exists on a `SCIM` connection.
        if (error && error.statusCode === 400 && ((_a = error.message) === null || _a === void 0 ? void 0 : _a.includes('already exists'))) {
            logger_1.default.warn(`SCIM configuration already exists on connection \"${connectionId}\".`);
            return null;
        }
        // Rate limiting errors should be mostly handled by `scimClient`. But, in the worst case scenario also we don;t want to break the pipeline.
        if (error && error.statusCode === 429) {
            logger_1.default.error(`The global rate limit has been exceeded, resulting in a ${error.statusCode} error. ${error.message}. Although this is an error, it is not blocking the pipeline.`);
            return null;
        }
        logger_1.default.error(`SCIM request failed with status code ${error.statusCode}. ${error.message || error.toString()}.`);
        throw error;
    }
    /**
     * Creates a new `SCIM` configuration.
     */
    createScimConfiguration(_a, _b) {
        return __awaiter(this, arguments, void 0, function* ({ id }, 
        // eslint-disable-next-line camelcase
        { user_id_attribute, mapping }) {
            logger_1.default.debug(`Creating SCIM configuration on connection ${id}`);
            return this.withErrorHandling(() => __awaiter(this, void 0, void 0, function* () { return this.connectionsManager.createScimConfiguration({ id }, { user_id_attribute, mapping }); }), 'create', id);
        });
    }
    /**
     * Retrieves `SCIM` configuration of an enterprise connection.
     */
    getScimConfiguration(_a) {
        return __awaiter(this, arguments, void 0, function* ({ id }) {
            logger_1.default.debug(`Getting SCIM configuration from connection ${id}`);
            return this.withErrorHandling(() => __awaiter(this, void 0, void 0, function* () { return this.connectionsManager.getScimConfiguration({ id }); }), 'get', id);
        });
    }
    /**
     * Updates an existing `SCIM` configuration.
     */
    updateScimConfiguration(_a, _b) {
        return __awaiter(this, arguments, void 0, function* ({ id }, 
        // eslint-disable-next-line camelcase
        { user_id_attribute, mapping }) {
            logger_1.default.debug(`Updating SCIM configuration on connection ${id}`);
            return this.withErrorHandling(() => __awaiter(this, void 0, void 0, function* () { return this.connectionsManager.updateScimConfiguration({ id }, { user_id_attribute, mapping }); }), 'patch', id);
        });
    }
    /**
     * Deletes an existing `SCIM` configuration.
     */
    deleteScimConfiguration(_a) {
        return __awaiter(this, arguments, void 0, function* ({ id }) {
            logger_1.default.debug(`Deleting SCIM configuration on connection ${id}`);
            return this.withErrorHandling(() => __awaiter(this, void 0, void 0, function* () { return this.connectionsManager.deleteScimConfiguration({ id }); }), 'delete', id);
        });
    }
    updateOverride(requestParams, bodyParams) {
        return __awaiter(this, void 0, void 0, function* () {
            // Extract `scim_configuration` from `bodyParams`.
            // Remove `scim_configuration` from `bodyParams`, because `connections.update` doesn't accept it.
            const { scim_configuration: scimBodyParams } = bodyParams;
            delete bodyParams.scim_configuration;
            // First, update `connections`.
            const updated = yield this.connectionsManager.update(requestParams, bodyParams);
            const idMapEntry = this.idMap.get(requestParams.id);
            // Now, update `scim_configuration` inside the updated connection.
            // If `scim_configuration` exists in both local and remote -> updateScimConfiguration(...)
            // If `scim_configuration` exists in remote but local -> deleteScimConfiguration(...)
            // If `scim_configuration` exists in local but remote -> createScimConfiguration(...)
            if (idMapEntry === null || idMapEntry === void 0 ? void 0 : idMapEntry.scimConfiguration) {
                if (scimBodyParams) {
                    if (this.scimScopes.update) {
                        yield this.updateScimConfiguration(requestParams, scimBodyParams);
                    }
                }
                else if (this.config('AUTH0_ALLOW_DELETE')) {
                    if (this.scimScopes.delete) {
                        yield this.deleteScimConfiguration(requestParams);
                    }
                }
                else {
                    logger_1.default.warn(`Skipping DELETE scim_configuration on \"${requestParams.id}\". Enable deletes by setting \"AUTH0_ALLOW_DELETE\" to true in your config.`);
                }
            }
            else if (scimBodyParams && this.scimScopes.create) {
                yield this.createScimConfiguration(requestParams, scimBodyParams);
            }
            // Return response from connections.update(...).
            return updated;
        });
    }
    createOverride(bodyParams) {
        return __awaiter(this, void 0, void 0, function* () {
            // Extract `scim_configuration` from `bodyParams`.
            // Remove `scim_configuration` from `bodyParams`, because `connections.create` doesn't accept it.
            const { scim_configuration: scimBodyParams } = bodyParams;
            delete bodyParams.scim_configuration;
            // First, create the new `connection`.
            const { data } = yield this.connectionsManager.create(bodyParams);
            if (scimBodyParams && this.scimScopes.create) {
                // Now, create the `scim_configuration` for newly created `connection`.
                yield this.createScimConfiguration({ id: data.id }, scimBodyParams);
            }
            // Return response from connections.create(...).
            return data;
        });
    }
}
exports.default = ScimHandler;
//# sourceMappingURL=scimHandler.js.map